"""logging Formatters"""
import cgi
import logging

__all__ = ['XMLLayout']

# Level names differ slightly in log4j, see
# http://logging.apache.org/log4j/1.2/apidocs/org/apache/log4j/Level.html
LOG4J_LEVEL_MAP = dict(WARNING='WARN', CRITICAL='FATAL')


class XMLLayout(logging.Formatter):

    """Formats log Records as XML according to the `log4j XMLLayout
    <http://logging.apache.org/log4j/docs/api/org/apache/log4j/xml/
    XMLLayout.html>_`
    """

    def format(self, record):
        """Format the log record as XMLLayout XML"""
        levelname = LOG4J_LEVEL_MAP.get(record.levelname, record.levelname)
        event = dict(name=cgi.escape(record.name),
                     threadName=cgi.escape(record.threadName),
                     levelname=cgi.escape(levelname),
                     created=int(record.created * 1000))

        event['message'] = LOG4J_MESSAGE % escape_cdata(record.getMessage())

        # FIXME: Support an NDC somehow
        event['ndc'] = ''
        #ndc = self.get_ndc(record)
        #if ndc:
        #    event['ndc'] = LOG4J_NDC % escape_cdata(ndc)

        event['throwable'] = ''
        if record.exc_info:
            if not record.exc_text:
                record.exc_text = self.formatException(record.exc_info)
            event['throwable'] = (LOG4J_THROWABLE %
                                  escape_cdata(record.exc_text))

        location_info = dict(pathname=cgi.escape(record.pathname),
                             lineno=record.lineno,
                             module=cgi.escape(record.module), funcName='')
        if hasattr(record, 'funcName'):
            # >= Python 2.5
            location_info['funcName'] = cgi.escape(record.funcName)
        event['locationInfo'] = LOG4J_LOCATIONINFO % location_info

        return LOG4J_EVENT % event


def escape_cdata(cdata):
    """Escape XML CDATA content"""
    return cdata.replace(']]>', ']]>]]&gt;<![CDATA[')

# General logging information
LOG4J_EVENT = """\
<log4j:event logger="%(name)s"
    timestamp="%(created)i"
    level="%(levelname)s"
    thread="%(threadName)s">
%(message)s%(ndc)s%(throwable)s%(locationInfo)s</log4j:event>
"""

# The actual log message
LOG4J_MESSAGE = """\
    <log4j:message><![CDATA[%s]]></log4j:message>
"""

# log4j's 'Nested Diagnostic Context': additional, customizable information
# included with the log record
LOG4J_NDC = """\
    <log4j:ndc><![CDATA[%s]]></log4j:ndc>
"""

# Exception information, if exc_info was included with the record
LOG4J_THROWABLE = """\
    <log4j:throwable><![CDATA[%s]]></log4j:throwable>
"""

# Traceback information
LOG4J_LOCATIONINFO = """\
    <log4j:locationInfo class="%(module)s"
        method="%(funcName)s"
        file="%(pathname)s"
        line="%(lineno)d"/>
"""



import cgi
import logging
import sys
from StringIO import StringIO
try:
    import xml.etree.ElementTree as ET
except ImportError:
    import elementtree.ElementTree as ET

from xmllayout import XMLLayout
from xmllayout.formatters import escape_cdata

PY3 = sys.version_info >= (3,)

LOG4J_NS = 'http://jakarta.apache.org/log4j/'

log = logging.getLogger(__name__)
stream = StringIO()
handler = logging.StreamHandler(stream)
xmllayout = XMLLayout()
handler.setFormatter(xmllayout)
log.addHandler(handler)
log.setLevel(logging.DEBUG)

class ElvisException(Exception):
    def __init__(self, info):
        self.info = info

    def __str__(self):
        return "<ElvisException: %s (TCB because he's the king baby)>" % \
            self.info

def test_xmllayout():
    _test_output('')
    _test_output('hello')
    _test_output('hello world', level=logging.DEBUG)
    _test_output('hello world!', level=logging.WARN, log4jlevel='WARN')
    _test_output('hello, world!', level=logging.WARNING, log4jlevel='WARN')
    _test_output('hello, world!!', level=logging.CRITICAL, log4jlevel='FATAL')
    _test_output('<xml><something>&nbsp;Hi</something></xml>')
    _test_output("""\
{'CONTENT_LENGTH': '0',
 'CONTENT_TYPE': '',
 'HTTP_ACCEPT': '*/*',
 'HTTP_ACCEPT_ENCODING': 'gzip, deflate',
 'HTTP_ACCEPT_LANGUAGE': 'en',
 'HTTP_CONNECTION': 'keep-alive',
 'HTTP_COOKIE': ''
 'HTTP_HOST': 'bob.local:5000',
 'HTTP_USER_AGENT': 'Mozilla/5.0 (Macintosh; U; Intel Mac OS X; en) AppleWebKit/419.2.1 (KHTML, like Gecko) Safari/419.3',
 'PATH_INFO': '/hello',
 'QUERY_STRING': '',
 'REMOTE_ADDR': '192.168.1.111',
 'REQUEST_METHOD': 'GET',
 'SCRIPT_NAME': '',
 'SERVER_NAME': '0.0.0.0',
 'SERVER_PORT': '5000',
 'SERVER_PROTOCOL': 'HTTP/1.1',
 'paste.evalexception': <pylons.error.PylonsEvalException object at 0x8c75ccc>,
 'wsgi.errors': <open file '<stderr>', mode 'w' at 0x81280b0>,
 'wsgi.input': <socket._fileobject object at 0x8c7a48c length=0>,
 'wsgi.multiprocess': False,
 'wsgi.multithread': True,
 'wsgi.run_once': False,
 'wsgi.url_scheme': 'http',
 'wsgi.version': (1, 0),}
""")

def test_exceptions():
    try:
        raise ElvisException('dog')
    except ElvisException:
        exc_module = ''
        if PY3:
            exc_module = ElvisException.__module__ + '.'
        exc_msg = ("raise ElvisException('dog')\n%sElvisException: "
                   "<ElvisException: dog (TCB because he's the king baby)>" %
                   exc_module)
        _test_output('Elvis has left the building', exc_info=True,
                     exc_msg=exc_msg)

def get_output():
    output = stream.getvalue().rstrip()
    stream.truncate(0)
    return '<test xmlns:log4j="%s">%s</test>' % (LOG4J_NS, output)

def _test_output(message, level=logging.INFO, log4jlevel=None, exc_info=None,
                 exc_msg=None):
    if log4jlevel is None:
        log4jlevel = logging.getLevelName(level)
    log.log(level, message, **dict(exc_info=exc_info))
    output = get_output()
    tree = ET.XML(output)

    event = tree.find("{%s}event" % LOG4J_NS)
    xml_level = event.get('level')
    assert xml_level == log4jlevel, message
    xml_message = tree.findtext("{%s}event/{%s}message" % (LOG4J_NS, LOG4J_NS))
    assert escape_cdata(message) == xml_message, message

    if exc_info:
        xml_exc = tree.findtext("{%s}event/{%s}throwable" % (LOG4J_NS,
                                                             LOG4J_NS))
        assert escape_cdata(exc_msg) in xml_exc, message



import sys

from setuptools import setup, find_packages

version = '1.0'

extra = {}
if sys.version_info >= (3,):
    extra['use_2to3'] = True

setup(name='XMLLayout',
      version=version,
      description="Formats Python log messages as log4j XMLLayout XML",
      long_description="""\
XMLLayout provides a Python logging Formatter that formats log messages as XML,
according to `log4j's XMLLayout specification
<http://jakarta.apache.org/log4j/docs/api/org/apache/log4j/xml/XMLLayout.html>`_.

XMLLayout formatted log messages can be viewed and filtered within the
`Chainsaw <http://logging.apache.org/log4j/docs/chainsaw.html>`_ application
(see the example section below), part of the Java based log4j project.

This package also includes a RawSocketHandler -- like
logging.handler.SocketHandler, but sends the raw log message over the socket
instead of a pickled version. RawSocketHandler can be configured to send log
messages to Chainsaw directly over a socket.

For example: to forward log messages to Chainsaw, if it were listening on
localhost port 4448::

    import logging
    import xmllayout

    handler = xmllayout.RawSocketHandler('localhost', 4448)
    handler.setFormatter(xmllayout.XMLLayout())
    logging.root.addHandler(handler)
""",
      classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python :: Implementation :: Jython',
        'Topic :: System :: Logging'
        ],
      keywords='logging log4j',
      author='Philip Jenvey',
      author_email='pjenvey@groovie.org',
      url='http://pypi.python.org/pypi/XMLLayout',
      license='BSD',
      packages=find_packages(),
      include_package_data=True,
      zip_safe=False,
      test_suite='nose.collector',
      tests_require=['nose'],
      **extra
      )



"""logging Handlers"""
import logging.handlers

__all__ = ['RawSocketHandler']


class RawSocketHandler(logging.handlers.SocketHandler):
    """Logging Handler that writes log records to a streaming socket.

    Like ``logging.handlers.SocketHandler``, but writes the actual formatted
    log record (not a pickled version).
    """

    def emit(self, record):
        """Emit a record.

        Formats the record and writes it to the socket in binary format. If
        there is an error with the socket, silently drop the packet. If there
        was a problem with the socket, re-establishes the socket.
        """
        try:
            msg = self.format(record)
            self.send(msg)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)
